@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -22,7 +22,7 @@
 
 ===============================================================================
 
-Support functions for logging.
+**Support functions for logging.**
 
 See https://docs.python.org/3.4/howto/logging.html#library-config
 
@@ -30,29 +30,37 @@ USER CODE should use the following general methods.
 
 (a) Simple:
 
-    import logging
-    log = logging.getLogger(__name__)  # for your own logs
-    logging.basicConfig()
+    .. code-block:: python
+
+        import logging
+        log = logging.getLogger(__name__)  # for your own logs
+        logging.basicConfig()
 
 (b) More complex:
 
-    import logging
-    log = logging.getLogger(__name__)
-    logging.basicConfig(format=LOG_FORMAT, datefmt=LOG_DATEFMT,
-                        level=loglevel)
+    .. code-block:: python
+
+        import logging
+        log = logging.getLogger(__name__)
+        logging.basicConfig(format=LOG_FORMAT, datefmt=LOG_DATEFMT,
+                            level=loglevel)
 
 (c) Using colour conveniently:
 
-    import logging
-    mylogger = logging.getLogger(__name__)
-    rootlogger = logging.getLogger()
+    .. code-block:: python
 
-    from whisker.log import configure_logger_for_colour
-    configure_logger_for_colour(rootlogger)
+        import logging
+        mylogger = logging.getLogger(__name__)
+        rootlogger = logging.getLogger()
+
+        from whisker.log import configure_logger_for_colour
+        configure_logger_for_colour(rootlogger)
 
 
 LIBRARY CODE should use the following general methods.
 
+.. code-block:: python
+
     import logging
     log = logging.getLogger(__name__)
 
@@ -63,7 +71,7 @@ LIBRARY CODE should use the following general methods.
 
     # LIBRARY CODE SHOULD NOT ADD ANY OTHER HANDLERS; see above.
 
-DO NOT call this module "logging"! Many things may get confused
+DO NOT call this module "logging"! Many things may get confused.
 
 """
 
@@ -99,6 +107,19 @@ def get_monochrome_handler(
         with_process_id: bool = False,
         with_thread_id: bool = False,
         stream: TextIO = None) -> logging.StreamHandler:
+    """
+    Gets a monochrome log handler using a standard format.
+
+    Args:
+        extranames: additional names to append to the logger's name
+        with_process_id: include the process ID in the logger's name?
+        with_thread_id: include the thread ID in the logger's name?
+        stream: ``TextIO`` stream to send log output to
+
+    Returns:
+        the :class:`logging.StreamHandler`
+
+    """
     fmt = "%(asctime)s.%(msecs)03d"
     if with_process_id or with_thread_id:
         procinfo = []  # type: List[str]
@@ -120,6 +141,19 @@ def get_colour_handler(extranames: List[str] = None,
                        with_process_id: bool = False,
                        with_thread_id: bool = False,
                        stream: TextIO = None) -> logging.StreamHandler:
+    """
+    Gets a colour log handler using a standard format.
+
+    Args:
+        extranames: additional names to append to the logger's name
+        with_process_id: include the process ID in the logger's name?
+        with_thread_id: include the thread ID in the logger's name?
+        stream: ``TextIO`` stream to send log output to
+
+    Returns:
+        the :class:`logging.StreamHandler`
+
+    """
     fmt = "%(white)s%(asctime)s.%(msecs)03d"  # this is dim white = grey
     if with_process_id or with_thread_id:
         procinfo = []  # type: List[str]
@@ -150,8 +184,17 @@ def configure_logger_for_colour(logger: logging.Logger,
                                 with_thread_id: bool = False) -> None:
     """
     Applies a preconfigured datetime/colour scheme to a logger.
-    Should ONLY be called from the "if __name__ == 'main'" script:
-        https://docs.python.org/3.4/howto/logging.html#library-config
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
+
+    Args:
+        logger: logger to modify
+        level: log level to set
+        remove_existing: remove existing handlers from logger first?
+        extranames: additional names to append to the logger's name
+        with_process_id: include the process ID in the logger's name?
+        with_thread_id: include the thread ID in the logger's name?
     """
     if remove_existing:
         logger.handlers = []  # http://stackoverflow.com/questions/7484454
@@ -166,6 +209,17 @@ def configure_logger_for_colour(logger: logging.Logger,
 def main_only_quicksetup_rootlogger(level: int = logging.DEBUG,
                                     with_process_id: bool = False,
                                     with_thread_id: bool = False) -> None:
+    """
+    Quick function to set up the root logger for colour.
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
+
+    Args:
+        level: log level to set
+        with_process_id: include the process ID in the logger's name?
+        with_thread_id: include the thread ID in the logger's name?
+    """
     # Nasty. Only call from "if __name__ == '__main__'" clauses!
     rootlogger = logging.getLogger()
     configure_logger_for_colour(rootlogger, level, remove_existing=True,
@@ -179,7 +233,12 @@ def main_only_quicksetup_rootlogger(level: int = logging.DEBUG,
 # =============================================================================
 
 def remove_all_logger_handlers(logger: logging.Logger) -> None:
-    """Remove all handlers from a logger."""
+    """
+    Remove all handlers from a logger.
+
+    Args:
+        logger: logger to modify
+    """
     while logger.handlers:
         h = logger.handlers[0]
         logger.removeHandler(h)
@@ -188,9 +247,18 @@ def remove_all_logger_handlers(logger: logging.Logger) -> None:
 def reset_logformat(logger: logging.Logger,
                     fmt: str,
                     datefmt: str = '%Y-%m-%d %H:%M:%S') -> None:
-    """Create a new formatter and apply it to the logger."""
-    # logging.basicConfig() won't reset the formatter if another module
-    # has called it, so always set the formatter like this.
+    """
+    Create a new formatter and apply it to the logger.
+
+    :func:`logging.basicConfig` won't reset the formatter if another module
+    has called it, so always set the formatter like this.
+
+    Args:
+        logger: logger to modify
+        fmt: passed to the ``fmt=`` argument of :class:`logging.Formatter`
+        datefmt: passed to the ``datefmt=`` argument of
+            :class:`logging.Formatter`
+    """
     handler = logging.StreamHandler()
     formatter = logging.Formatter(fmt=fmt, datefmt=datefmt)
     handler.setFormatter(formatter)
@@ -202,8 +270,15 @@ def reset_logformat(logger: logging.Logger,
 def reset_logformat_timestamped(logger: logging.Logger,
                                 extraname: str = "",
                                 level: int = logging.INFO) -> None:
-    """Apply a simple time-stamped log format to an existing logger, and set
-    its loglevel to either DEBUG or INFO."""
+    """
+    Apply a simple time-stamped log format to an existing logger, and set
+    its loglevel to either ``logging.DEBUG`` or ``logging.INFO``.
+
+    Args:
+        logger: logger to modify
+        extraname: additional name to append to the logger's name
+        level: log level to set
+    """
     namebit = extraname + ":" if extraname else ""
     fmt = ("%(asctime)s.%(msecs)03d:%(levelname)s:%(name)s:" + namebit +
            "%(message)s")
@@ -220,9 +295,15 @@ def reset_logformat_timestamped(logger: logging.Logger,
 def configure_all_loggers_for_colour(remove_existing: bool = True) -> None:
     """
     Applies a preconfigured datetime/colour scheme to ALL logger.
-    Should ONLY be called from the "if __name__ == 'main'" script:
-        https://docs.python.org/3.4/howto/logging.html#library-config
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
+
     Generally MORE SENSIBLE just to apply a handler to the root logger.
+
+    Args:
+        remove_existing: remove existing handlers from logger first?
+
     """
     handler = get_colour_handler()
     apply_handler_to_all_logs(handler, remove_existing=remove_existing)
@@ -232,9 +313,15 @@ def apply_handler_to_root_log(handler: logging.Handler,
                               remove_existing: bool = False) -> None:
     """
     Applies a handler to all logs, optionally removing existing handlers.
-    Should ONLY be called from the "if __name__ == 'main'" script:
-        https://docs.python.org/3.4/howto/logging.html#library-config
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
+
     Generally MORE SENSIBLE just to apply a handler to the root logger.
+
+    Args:
+        handler: the handler to apply
+        remove_existing: remove existing handlers from logger first?
     """
     rootlog = logging.getLogger()
     if remove_existing:
@@ -246,9 +333,15 @@ def apply_handler_to_all_logs(handler: logging.Handler,
                               remove_existing: bool = False) -> None:
     """
     Applies a handler to all logs, optionally removing existing handlers.
-    Should ONLY be called from the "if __name__ == 'main'" script:
-        https://docs.python.org/3.4/howto/logging.html#library-config
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
+
     Generally MORE SENSIBLE just to apply a handler to the root logger.
+
+    Args:
+        handler: the handler to apply
+        remove_existing: remove existing handlers from logger first?
     """
     # noinspection PyUnresolvedReferences
     for name, obj in logging.Logger.manager.loggerDict.items():
@@ -262,8 +355,9 @@ def copy_root_log_to_file(filename: str,
                           datefmt: str = LOG_DATEFMT) -> None:
     """
     Copy all currently configured logs to the specified file.
-    Should ONLY be called from the "if __name__ == 'main'" script:
-        https://docs.python.org/3.4/howto/logging.html#library-config
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
     """
     fh = logging.FileHandler(filename)
     # default file mode is 'a' for append
@@ -277,8 +371,15 @@ def copy_all_logs_to_file(filename: str,
                           datefmt: str = LOG_DATEFMT) -> None:
     """
     Copy all currently configured logs to the specified file.
-    Should ONLY be called from the "if __name__ == 'main'" script:
-        https://docs.python.org/3.4/howto/logging.html#library-config
+
+    Should ONLY be called from the ``if __name__ == 'main'`` script;
+    see https://docs.python.org/3.4/howto/logging.html#library-config.
+
+    Args:
+        filename: file to send log output to
+        fmt: passed to the ``fmt=`` argument of :class:`logging.Formatter`
+        datefmt: passed to the ``datefmt=`` argument of
+            :class:`logging.Formatter`
     """
     fh = logging.FileHandler(filename)
     # default file mode is 'a' for append
@@ -289,8 +390,10 @@ def copy_all_logs_to_file(filename: str,
 
 # noinspection PyProtectedMember
 def get_formatter_report(f: logging.Formatter) -> Optional[Dict[str, str]]:
-    """Returns information on a log formatter, as a dictionary.
-    For debugging."""
+    """
+    Returns information on a log formatter, as a dictionary.
+    For debugging.
+    """
     if f is None:
         return None
     return {
@@ -301,7 +404,10 @@ def get_formatter_report(f: logging.Formatter) -> Optional[Dict[str, str]]:
 
 
 def get_handler_report(h: logging.Handler) -> Dict[str, Any]:
-    """Returns information on a log handler, as a dictionary. For debugging."""
+    """
+    Returns information on a log handler, as a dictionary.
+    For debugging.
+    """
     return {
         'get_name()': h.get_name(),
         'level': h.level,
@@ -312,7 +418,9 @@ def get_handler_report(h: logging.Handler) -> Dict[str, Any]:
 
 def get_log_report(log: Union[logging.Logger,
                               logging.PlaceHolder]) -> Dict[str, Any]:
-    """Returns information on a log, as a dictionary. For debugging."""
+    """
+    Returns information on a log, as a dictionary. For debugging.
+    """
     if isinstance(log, logging.Logger):
         # suppress invalid error for Logger.manager:
         # noinspection PyUnresolvedReferences
@@ -335,7 +443,7 @@ def get_log_report(log: Union[logging.Logger,
 
 def print_report_on_all_logs() -> None:
     """
-    Use print() to report information on all logs.
+    Use :func:`print` to report information on all logs.
     """
     d = {}
     # noinspection PyUnresolvedReferences
@@ -348,6 +456,13 @@ def print_report_on_all_logs() -> None:
 
 def set_level_for_logger_and_its_handlers(log: logging.Logger,
                                           level: int) -> None:
+    """
+    Set a log level for a log and all its handlers.
+
+    Args:
+        log: log to modify
+        level: log level to set
+    """
     log.setLevel(level)
     for h in log.handlers:  # type: logging.Handler
         h.setLevel(level)
@@ -358,6 +473,9 @@ def set_level_for_logger_and_its_handlers(log: logging.Logger,
 # =============================================================================
 
 class HtmlColorFormatter(logging.Formatter):
+    """
+    Class to format Python logs in coloured HTML.
+    """
     log_colors = {
         logging.DEBUG: '#008B8B',  # dark cyan
         logging.INFO: '#00FF00',  # green
@@ -375,7 +493,13 @@ class HtmlColorFormatter(logging.Formatter):
 
     def __init__(self, append_br: bool = False,
                  replace_nl_with_br: bool = True) -> None:
-        # https://hg.python.org/cpython/file/3.5/Lib/logging/__init__.py
+        r"""
+        Args:
+            append_br: append ``<br>`` to each line?
+            replace_nl_with_br: replace ``\n`` with ``<br>`` in messages?
+
+        See https://hg.python.org/cpython/file/3.5/Lib/logging/__init__.py
+        """
         super().__init__(
             fmt='%(message)s',
             datefmt='%Y-%m-%d %H:%M:%S',
@@ -385,8 +509,11 @@ class HtmlColorFormatter(logging.Formatter):
         self.replace_nl_with_br = replace_nl_with_br
 
     def format(self, record: logging.LogRecord) -> str:
-        # record is a LogRecord
-        # https://docs.python.org/3.4/library/logging.html#logging.LogRecord
+        """
+        Internal function to format the :class:`LogRecord` as HTML.
+
+        See https://docs.python.org/3.4/library/logging.html#logging.LogRecord
+        """
 
         # message = super().format(record)
         super().format(record)
@@ -418,11 +545,14 @@ class HtmlColorFormatter(logging.Formatter):
 
 
 # =============================================================================
-# HTML handler (using HtmlColorFormatter) that sends output to a function,
-# e.g. for display in a Qt window
+# HtmlColorHandler
 # =============================================================================
 
 class HtmlColorHandler(logging.StreamHandler):
+    """
+    HTML handler (using :class:`HtmlColorFormatter`) that sends output to a
+    function, e.g. for display in a Qt window
+    """
     def __init__(self, logfunction: Callable[[str], None],
                  level: int = logging.INFO) -> None:
         super().__init__()
@@ -431,6 +561,9 @@ class HtmlColorHandler(logging.StreamHandler):
         self.setLevel(level)
 
     def emit(self, record: logging.LogRecord) -> None:
+        """
+        Internal function to process a :class:`LogRecord`.
+        """
         # noinspection PyBroadException
         try:
             html = self.format(record)
@@ -452,6 +585,11 @@ class HtmlColorHandler(logging.StreamHandler):
 # - https://www.simonmweber.com/2014/11/24/python-logging-traps.html
 
 class BraceMessage(object):
+    """
+    Class to represent a message that includes a message including braces
+    (``{}``) and a set of ``args``/``kwargs``. When converted to a ``str``,
+    the message is realized via ``msg.format(*args, **kwargs)``.
+    """
     def __init__(self,
                  fmt: str,
                  args: Tuple[Any, ...],
@@ -476,7 +614,7 @@ class BraceStyleAdapter(logging.LoggerAdapter):
                  pass_special_logger_args: bool = True,
                  strip_special_logger_args_from_fmt: bool = False) -> None:
         """
-        Wraps a logger so we can use {}-style string formatting.
+        Wraps a logger so we can use ``{}``-style string formatting.
 
         Args:
             logger:
@@ -490,7 +628,22 @@ class BraceStyleAdapter(logging.LoggerAdapter):
                 If we're passing special arguments to the logger, should we
                 remove them from the argments passed to the string formatter?
                 There is no obvious cost to saying no.
-        """
+                
+        Specimen use:
+        
+        .. code-block:: python
+        
+            import logging
+            from cardinal_pythonlib.logs import BraceStyleAdapter, main_only_quicksetup_rootlogger 
+            
+            log = BraceStyleAdapter(logging.getLogger(__name__))
+            
+            main_only_quicksetup_rootlogger(level=logging.DEBUG)
+            
+            log.info("Hello {}, {title} {surname}!", "world", title="Mr", surname="Smith") 
+            # 2018-09-17 16:13:50.404 __main__:INFO: Hello world, Mr Smith!
+        
+        """  # noqa
         super().__init__(logger=logger, extra=None)
         self.pass_special_logger_args = pass_special_logger_args
         self.strip_special_logger_args_from_fmt = strip_special_logger_args_from_fmt  # noqa
@@ -546,6 +699,9 @@ class BraceStyleAdapter(logging.LoggerAdapter):
 # =============================================================================
 
 if __name__ == '__main__':
+    """
+    Command-line validation checks.
+    """
     main_only_quicksetup_rootlogger(logging.INFO)
     _log = BraceStyleAdapter(logging.getLogger(__name__))
     _log.info("1. Hello!")
